@@ -72,6 +72,10 @@ struct Message::Private {
 
 	Option<std::string> language; /* body ISO language code */
 
+	// after unload_mime_message, we don't need to re-parse the message.
+	// issue #2802.
+	bool document_filled{};
+
 private:
 	Document::Options doc_opts(Message::Options mopts) {
 		return any_of(opts & Message::Options::SupportNgrams) ?
@@ -492,11 +496,15 @@ maybe_handle_pkcs7(const MimeObject& obj, Message::Private& info)
 #pragma GCC diagnostic push
 #pragma GCC diagnostic ignored "-Wswitch-enum"
 		// CompressedData, CertsOnly, Unknown
-		switch (smime.smime_type()) {
+		const auto smtype{smime.smime_type()};
+		switch (smtype) {
 		case Mu::MimeApplicationPkcs7Mime::SecureMimeType::SignedData:
 			info.flags |= Flags::Signed;
 			break;
 		case Mu::MimeApplicationPkcs7Mime::SecureMimeType::EnvelopedData:
+		case Mu::MimeApplicationPkcs7Mime::SecureMimeType::Unknown:
+			/* The "Unknown" case... GMIME doesn't give us anything
+			 * more specific, so assume it's encrypted for now  */
 			info.flags |= Flags::Encrypted;
 			break;
 		default:
@@ -671,6 +679,10 @@ doc_add_reply_to(Document& doc, const MimeMessage& mime_msg)
 static void
 fill_document(Message::Private& priv)
 {
+	if (priv.document_filled) {
+		return; // nothing to do.
+	}
+
 	/* hunt & gather info from message tree */
 	Document& doc{priv.doc};
 	MimeMessage& mime_msg{priv.mime_msg.value()};
@@ -687,7 +699,7 @@ fill_document(Message::Private& priv)
 	doc_add_list_post(doc, mime_msg); /* only in sexp */
 	doc_add_reply_to(doc, mime_msg);  /* only in sexp */
 
-	field_for_each([&](auto&& field) {
+	field_for_each([&](const auto& field) {
 		/* insist on explicitly handling each */
 #pragma GCC diagnostic push
 #pragma GCC diagnostic error "-Wswitch"
@@ -771,6 +783,8 @@ fill_document(Message::Private& priv)
 #pragma GCC diagnostic pop
 
 	});
+
+	priv.document_filled = true;
 }
 
 Option<std::string>
@@ -867,6 +881,14 @@ Message::update_after_move(const std::string& new_path,
 	priv_->doc.add(Field::Id::Path, new_path);
 	priv_->doc.add(Field::Id::Changed, priv_->ctime);
 
+	// note: content-flags are retained from the existing; the unread flag
+	// is implied. only file-flags are allowed for new_flags; anything else
+	// is filtered-out
+
+	new_flags = flags_maildir_file(new_flags);
+	new_flags |= flags_filter(flags(), MessageFlagCategory::Content);
+	new_flags = imply_unread(new_flags);
+
 	set_flags(new_flags);
 
 	if (const auto res = set_maildir(sanitize_maildir(new_maildir)); !res)
