@@ -1,5 +1,5 @@
  /*
-** Copyright (C) 2008-2024 Dirk-Jan C. Binnema <djcb@djcbsoftware.nl>
+** Copyright (C) 2008-2025 Dirk-Jan C. Binnema <djcb@djcbsoftware.nl>
 **
 ** This program is free software; you can redistribute it and/or modify it
 ** under the terms of the GNU General Public License as published by the
@@ -164,6 +164,10 @@ get_query(const Store& store, const Options& opts)
 static Result<void>
 prepare_links(const Options& opts)
 {
+	// XXX: can this be done as part of the option-parsing?
+	if (opts.find.linksdir.empty())
+		return Err(Error::Code::InvalidArgument, "--linksdir is required");
+
 	/* note, mu_maildir_mkdir simply ignores whatever part of the
 	 * mail dir already exists */
 	if (auto&& res = maildir_mkdir(opts.find.linksdir, 0700, true); !res)
@@ -343,7 +347,7 @@ output_plain(const Option<Message>& msg, const OutputInfo& info,
 
 	output_plain_fields(*msg, opts.find.fields, !opts.nocolor, opts.find.threads);
 
-	if (opts.view.summary_len)
+	if (opts.find.summary_len)
 		print_summary(*msg, opts);
 
 	return Ok();
@@ -379,7 +383,10 @@ output_json(const Option<Message>& msg, const OutputInfo& info, const Options& o
 	if (!msg)
 		return Ok();
 
-	mu_println("{}{}", msg->sexp().to_json_string(), info.last ? "" : ",");
+	const Sexp::Format frm{opts.find.format == Format::Json2 ?  Sexp::Format::NoColon :
+		Sexp::Format::Default};
+
+	mu_println("{}{}", msg->sexp().to_json_string(frm), info.last ? "" : ",");
 
 	return Ok();
 }
@@ -439,6 +446,7 @@ get_output_func(const Options& opts)
 	case Format::Sexp:
 		return output_sexp;
 	case Format::Json:
+	case Format::Json2:
 		return output_json;
 	default:
 		throw Error(Error::Code::Internal,
